<?php

namespace Test\GabrielGciaFdez\Service;

use GabrielGciaFdez\Service\PhpArchExampleValidator;

class PhpArchExampleValidatorTest extends \PHPUnit_Framework_TestCase
{
    /**
     * @dataProvider emailDataProvider
     */
    public function testIsValid($expected, $jsonEncodedData)
    {
        $validatorService = new PhpArchExampleValidator();
        $data = json_decode($jsonEncodedData, true);
        $this->assertEquals($expected, $validatorService->isValid($data), $validatorService->getErrorMessage());
    }

    /**
     * @expectedException \InvalidArgumentException
     * @expectedExceptionMessage Data cannot be null
     */
    public function testAnInvalidArgumentExceptionIsThrownIfDataIsNull()
    {
        $validatorService = new PhpArchExampleValidator();
        $validatorService->isValid(null);
    }

    /**
     * @expectedException Symfony\Component\Validator\Exception\UnexpectedTypeException
     * @expectedExceptionMessage Expected argument of type "array or \Countable", "string" given
     */
    public function testAnUnexpectedTypeExceptionIsThrownIfRecipientsIsNotAnArray()
    {
        $date = new \DateTime('-1 hour');
        $strDate = $date->format('Y-m-d H:i:s');

        $validatorService = new PhpArchExampleValidator();
        $data = json_decode('{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": "this must be an array",
                "date": "'. $strDate .'",
                "hasLinks": true,
                "numOfLinks": 2
            }', true);

        $validatorService->isValid($data);
    }

    public function testDataIsNotValidWithAThreeHoursAgoDateTime()
    {
        $date = new \DateTime('-3 hour');
        $strDate = $date->format('Y-m-d H:i:s');

        $validatorService = new PhpArchExampleValidator();
        $data = json_decode('{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": true,
                "numOfLinks": 2
            }', true);

        $this->assertFalse($validatorService->isValid($data))   ;
    }

    public function emailDataProvider()
    {
        $date = new \DateTime('-1 hour');
        $strDate = $date->format('Y-m-d H:i:s');

        return array(
            // return true with all valid fields
            array(true, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return true with userHash set to null and a valid sender
            array(true, '{
                "userHash": null,
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return true with sender set to null and a valid userHash
            array(true, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": null,
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return false with sender and userHash set to null
            array(false, '{
                "userHash": null,
                "sender": null,
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return false with an invalid userHash
            array(false, '{
                "userHash": "invalid user hash",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return false with an invalid sender email address
            array(false, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "this is not @ email . address",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return false with no subject field
            array(false, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return false with a null subject
            array(false, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": null,
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return false with a long subject
            array(false, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "Lorem ipsum dolo sit amet dolo Lorem ipsum dolo sit amet dolo Lorem ipsum dolo sit amet dolo",
                "subject": null,
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return false with an empty recipients array
            array(false, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [],
                "date": "'. $strDate .'",
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return false with a recipients array without email
            array(false, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe"
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return false with a recipients array without name
            array(false, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "email": "jdoe@domain.com"
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return false with a recipients array with an invalid email
            array(false, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@.com"
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return false with a recipients array with an invalid name
            array(false, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": 1,
                        "email": "jdoe@domain.com"
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return false without date field
            array(false, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return false with an invalid date string
            array(false, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "1st May 2014 15:52:00",
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return false with an invalid date in the future
            array(false, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "2222-01-01 00:00:00",
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return false with an invalid date too old
            array(false, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "1900-01-01 00:00:00",
                "hasLinks": true,
                "numOfLinks": 2
            }'),
            // return false without hasLinks field
            array(false, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "'. $strDate .'",
                "numOfLinks": 2
            }'),
            // return false with hasLinks field not boolean
            array(false, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": "notBooleanValue",
                "numOfLinks": 2
            }'),
            // return true without numOfLinks field and hasLinks set to false
            array(true, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": false
            }'),
            // return false without numOfLinks field and hasLinks set to true
            array(false, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": true
            }'),
            // return false with non-integer numOfLinks value
            array(false, '{
                "userHash": "18a98c35f49808b45edadc75fb1b25ebfd4037d6",
                "sender": "hi@gabi.cat",
                "subject": "Validation rocks!",
                "recipients": [
                    {
                        "name": "John Doe",
                        "email": "jdoe@email.com"
                    },
                    {
                        "email": "other@email.com",
                        "name": null
                    }
                ],
                "date": "'. $strDate .'",
                "hasLinks": true,
                "numOfLinks": -33
            }'),
        );
    }
}